package sl12tutorial;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

public class Cascade extends HttpServlet
{
  static final String NS_SL_ = "http://www.buergerkarte.at/namespaces/securitylayer/1.2#"; 

  static final String BKU_URL_ = "http://127.0.0.1:3495/http-security-layer-request";
//  static final String BKU_URL_ = "http://127.0.0.1:13495/http-security-layer-request";

  public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    doGet(request, response);
  }
  
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    // Prfen, woher der Aufruf kommt
    String useParam = request.getParameter("use"); 
    try
    {
	    if (useParam != null && "available".equals(useParam))
	    {
	      // Aufruf kommt von BKU und enthlt InfoboxAvailableResponse, Response auswerten und InfoboxCreate bzw.
	      // InfoboxUpdate senden
	      processAvailable(request, response);
	    }
	    else if (useParam != null && "update".equals(useParam))
	    {
	      // Aufruf kommt von BKU und enthlt InfoboxCreateResponse, InfoboxUpdate senden
	      processUpdate(request, response);
	    }
	    else if (useParam != null && "done".equals(useParam))
	    {
	      // Aufruf kommt von BKU und enthlt InfoboxUpdateResponse, finale Antwortseite senden
	      processDone(request, response);
	    }
	    else
	    {
	      // Aufruf kommt nicht von BKU, Startseite erzeugen und senden
	      processInitial(request, response);
	    }
    }
    catch (Exception e)
    {
      if (!(e instanceof IOException)) throw new IOException(e.getMessage());
    }
  }
  
  private void processInitial(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    String servletURL = request.getRequestURL().toString();
//    String servletURL = "http://localhost:18080/SL12Tutorial/Cascade";

    // Startseite erzeugen und senden
    response.setStatus(200);
    response.setContentType("text/html");
    printForm(response, servletURL);
  }
  
  private void processAvailable(HttpServletRequest request, HttpServletResponse response)
    throws Exception
  { 
    String servletURL = request.getRequestURL().toString();
//    String servletURL = "http://localhost:18080/SL12Tutorial/Cascade";
    
    String xMLResponse = request.getParameter("XMLResponse");
    Document xMLResponseDoc = parseDocument(xMLResponse);
    boolean available = isInfoboxAvailable(xMLResponseDoc);
      
    if (available)
    {
      // Infobox aktualisieren
      processUpdate(request, response);
    }
    else
    {
      // Infobox anlegen
      response.setStatus(307);
      response.setContentType("text/xml");
      response.setHeader("Location", servletURL + "?use=update");
      printCreateRequest(response);
    }
  }
  
  private void processUpdate(HttpServletRequest request, HttpServletResponse response)
    throws Exception
  {
    String servletURL = request.getRequestURL().toString();
//  String servletURL = "http://localhost:18080/SL12Tutorial/Cascade";

    // Infobox aktualisieren
    response.setStatus(307);
    response.setContentType("text/xml");
    response.setHeader("Location", servletURL + "?use=done");
    printUpdateRequest(response);
  }
  
  private void processDone(HttpServletRequest request, HttpServletResponse response)
    throws Exception
	{
	  // Finale Antwortseite erzeugen und senden
	  response.setStatus(200);
	  response.setContentType("text/html");
	  
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head><title>Kaskadierung: Abschluss</title></head>");
    out.println("<body>");
    out.println("<p>Infobox <code style='background-color: silver;'>TutorialBinary</code> ist angelegt und auf dem letzten Stand.</p>");
    out.println("</body>");
    out.println("</html>");
	  
	}

  private boolean isInfoboxAvailable(Document xMLResponseDoc)
  {
    // Prfen, ob Infobox "TutorialBinary" vorhanden ist
    NodeList infoboxes = xMLResponseDoc.getDocumentElement().getElementsByTagNameNS(NS_SL_, "InfoboxIdentifier");
    boolean available = false;
    for (int i = 0; i < infoboxes.getLength(); i++)
    {
      String infoboxId = infoboxes.item(i).getFirstChild().getNodeValue().trim();
      if ("TutorialBinary".equals(infoboxId))
      {
        available = true;
        break;
      }
    }
    return available;
  }

  private void printForm(HttpServletResponse response, String servletURL) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head><title>Kaskadierung: Start</title></head>");
    out.println("<body>");
    out.println("<form method=\"post\" action=\"" + BKU_URL_ + "\">");
    out.println("<input name=\"XMLRequest\" type=\"hidden\" value=\"&lt;?xml version='1.0' encoding='UTF-8'?>&lt;InfoboxAvailableRequest xmlns='http://www.buergerkarte.at/namespaces/securitylayer/1.2#'/>\"/>");
    out.println("<input name=\"DataURL\" type=\"hidden\" value=\"" + servletURL + "?use=available" + "\"/>");
    out.println("<input name=\"Request absenden\" type=\"submit\"/>");
    out.println("</form>");
    out.println("</body>");
    out.println("</html>");
  }

  private void printCreateRequest(HttpServletResponse response) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.write("<?xml version='1.0' encoding='UTF-8'?>\n".getBytes("UTF-8"));
    out.write("<sl:InfoboxCreateRequest xmlns:sl='http://www.buergerkarte.at/namespaces/securitylayer/1.2#'>\n".getBytes("UTF-8"));
    out.write("  <sl:InfoboxIdentifier>TutorialBinary</sl:InfoboxIdentifier>\n".getBytes("UTF-8"));
    out.write("  <sl:InfoboxType>BinaryFile</sl:InfoboxType>\n".getBytes("UTF-8"));
    out.write("  <sl:Creator>Tutorium zur sterreichischen Brgerkarte</sl:Creator>\n".getBytes("UTF-8"));
    out.write("  <sl:Purpose>Demonstriert das Anlegen, Lesen, Verndern und Lschen einer Binrdatei.</sl:Purpose>\n".getBytes("UTF-8"));
    out.write("</sl:InfoboxCreateRequest>\n".getBytes("UTF-8"));
  }

  private void printUpdateRequest(HttpServletResponse response) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<?xml version='1.0' encoding='UTF-8'?>");
    out.println("<sl:InfoboxUpdateRequest xmlns:sl='http://www.buergerkarte.at/namespaces/securitylayer/1.2#'>");
    out.println("  <sl:InfoboxIdentifier>TutorialBinary</sl:InfoboxIdentifier>");
    out.println("  <sl:BinaryFileParameters>");
    out.println("    <sl:XMLContent>");
    out.println("      <my:Customer xmlns:my='urn:my.namespace'>");
    out.println("        <my:Name>Tassilo Tester</my:Name>");
    out.println("        <my:LastVisit>2005-01-01</my:LastVisit>");
    out.println("      </my:Customer>");
    out.println("    </sl:XMLContent>");
    out.println("  </sl:BinaryFileParameters>");
    out.println("</sl:InfoboxUpdateRequest>");
  }

  private Document parseDocument(String docStr) throws Exception 
  {
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    dbf.setNamespaceAware(true);
    DocumentBuilder db = dbf.newDocumentBuilder();
    ByteArrayInputStream docBAIS = new ByteArrayInputStream(docStr.getBytes());
    Document doc = db.parse(docBAIS);
    return doc;
  }
}
