package sl12tutorial;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.fileupload.DiskFileUpload;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class SignAttachments extends HttpServlet
{
  static final String NS_SL_ = "http://www.buergerkarte.at/namespaces/securitylayer/1.2#"; 

   static final String BKU_URL_ = "http://127.0.0.1:3495/http-security-layer-request";
//  static final String BKU_URL_ = "http://127.0.0.1:13495/http-security-layer-request";

  public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    doGet(request, response);
  }
  
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    // Prfen, woher Aufruf kommt
    String useParam = request.getParameter("use"); 
    try
    {
	    if (useParam != null && "sign".equals(useParam))
	    {
	      // Aufruf kommt von BKU und enthlt CreateHashResponse, Hashwert auswerten und CreateXMLSignature senden
	      processSign(request, response);
	    }
	    else if (useParam != null && "done".equals(useParam))
	    {
	      // Aufruf kommt von BKU und enthlt CreateXMLSignatureResponse, finale Antwortseite erzeugen und senden
	      processDone(request, response);
	    }
	    else
	    {
	      // Aufruf kommt nicht von BKU, Startseite erzeugen und senden
	      request.getSession().invalidate();
	      processInitial(request, response);
	    }
    }
    catch (Exception e)
    {
      if (!(e instanceof IOException)) throw new IOException(e.getMessage());
    }
  }
  
  private void processInitial(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    String servletURL = request.getRequestURL().toString();
//  String SERVLET_URL_ = "http://localhost:18080/SL12Tutorial/SignAttachments";
 
    // Neue Session beginnen
    HttpSession session = request.getSession(true);
    
    // Formular mit Befehl zum Berechnen des Hashwerts an Browser senden
    response.setStatus(200);
    response.setContentType("text/html");
    printForm(response, session, servletURL);
  }
  
  private void processSign(HttpServletRequest request, HttpServletResponse response)
    throws Exception
  {
    String servletURL = request.getRequestURL().toString();
//  String SERVLET_URL_ = "http://localhost:18080/SL12Tutorial/SignAttachments";
 
    // Session abrufen
    HttpSession session = request.getSession();
    
    // Parse den Inhalt des Http-Requests (multipart/formdata)
    Iterator itemsIt = parseRequestAttrs(request);
    String xMLResponse = null, firstName = null, lastName = null;
    InputStream attachment = null;
    while (itemsIt.hasNext())
    {
      FileItem currItem = (FileItem) itemsIt.next();
      String currItemName = currItem.getFieldName();
      if ("XMLResponse".equals(currItemName)) xMLResponse = currItem.getString();
      else if ("Vorname_".equals(currItemName)) firstName = currItem.getString();
      else if ("Nachname_".equals(currItemName)) lastName = currItem.getString();
      else if ("Beilage_".equals(currItemName)) attachment = currItem.getInputStream();
    }
    
    // Speichere Beilagen-Attachment in Session
    session.setAttribute("Beilage_", attachment);
    
    // Extrahiere Hashwert aus XML-Response
    Document xMLResponseDoc = parseDocument(xMLResponse);
    Element xMLResponseRoot = xMLResponseDoc.getDocumentElement();
    Element hashValueElem = (Element)xMLResponseRoot.getElementsByTagNameNS(NS_SL_, "HashValue").item(0);
    String hashValue = hashValueElem.getFirstChild().getNodeValue();

    // Prfen, ob Hashwert zur Beilage passt, wird hier nicht nher ausgefhrt
    
    // Generiere die zu signierenden Daten
    String data2BSigned = createData2BSigned(firstName, lastName, hashValue);
    
    // Befehl zum Signieren an BKU senden
    response.setStatus(307);
    response.setContentType("text/xml");
    response.setHeader("Location", encodeURL(servletURL + "?use=done", session));
    printSignRequest(response, data2BSigned);
  }
  
  private void processDone(HttpServletRequest request, HttpServletResponse response)
    throws Exception
	{
    // Session abrufen
    HttpSession session = request.getSession();

    // Parse den Inhalt des Http-Requests (multipart/formdata)
    Iterator itemsIt = parseRequestAttrs(request);
    String xMLResponse = null;
    InputStream attachment = null;
    while (itemsIt.hasNext())
    {
      FileItem currItem = (FileItem) itemsIt.next();
      String currItemName = currItem.getFieldName();
      if ("XMLResponse".equals(currItemName)) xMLResponse = currItem.getString();
    }
    Document xMLResponseDoc = parseDocument(xMLResponse);

	  // XML-Signatur in Session speichern
    Element xMLResponseRoot = xMLResponseDoc.getDocumentElement();
    Element xMLSignature = (Element)xMLResponseRoot.getElementsByTagNameNS("*", "*").item(0);
    session.setAttribute("XMLSignature", xMLSignature);

    // Signatur prfen, wird hier nicht nher ausgefhrt
    
    // Signatur und Beilage verarbeiten, wird hier nicht nher ausgefhrt
    
	  // Abschluss
    String signatureStr = serializeElement(xMLSignature);

    response.setStatus(200);
	  response.setContentType("text/html");
    printFinalResponse(response, signatureStr);
	  
    // Session terminieren
    session.invalidate();
	}

  private void printForm(HttpServletResponse response, HttpSession session, String servletURL) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.write("<?xml version='1.0' encoding='UTF-8'?>\n".getBytes("UTF-8"));
    out.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n".getBytes("UTF-8"));
    out.write("<html xmlns='http://www.w3.org/1999/xhtml'>\n".getBytes("UTF-8"));
    out.write("	<head>\n".getBytes("UTF-8"));
    out.write("		<title>Signieren von Beilagen: Start</title>\n".getBytes("UTF-8"));
    out.write("	</head>\n".getBytes("UTF-8"));
    out.write("	<body>\n".getBytes("UTF-8"));
    out.write("		<p>Bitte f&uuml;llen Sie das folgende Formular 0815 aus:</p>\n".getBytes("UTF-8"));
    out.write(("		<form style='background-color: silver;' action='" + BKU_URL_ + "' method='post' enctype='multipart/form-data' name='form1' id='form1'>\n").getBytes("UTF-8"));
    out.write("		  <p><code>Vorname: </code><input name='Vorname_' type='text' id='vorname_'/><br/>\n".getBytes("UTF-8"));
    out.write("		  <code>Nachname:</code><input name='Nachname_' type='text' id='nachname_'/></p>	\n".getBytes("UTF-8"));
    out.write("		  <p><code>Lageplan:</code><input type='file' name='Beilage_'/></p>\n".getBytes("UTF-8"));
    out.write("		  <p><input type='submit' value='Weiter ...'/></p>\n".getBytes("UTF-8"));
    out.write("		  <input type='hidden' name='XMLRequest' value='&lt;?xml version=\"1.0\" encoding=\"UTF-8\"?>&lt;sl:CreateHashRequest xmlns:sl=\"http://www.buergerkarte.at/namespaces/securitylayer/1.2#\">  &lt;sl:HashInfo RespondHashData=\"false\">    &lt;sl:HashData>      &lt;sl:MetaInfo>        &lt;sl:MimeType>application/octet-stream&lt;/sl:MimeType>      &lt;/sl:MetaInfo>      &lt;sl:Content Reference=\"formdata:Beilage_\"/>    &lt;/sl:HashData>    &lt;sl:HashAlgorithm>http://www.w3.org/2000/09/xmldsig#sha1&lt;/sl:HashAlgorithm>    &lt;sl:FriendlyName>Beilage zum Formular 0815&lt;/sl:FriendlyName>  &lt;/sl:HashInfo>&lt;/sl:CreateHashRequest>'/>\n".getBytes("UTF-8"));
    out.write(("     <input name='DataURL' type='hidden' value='" + encodeURL(servletURL + "?use=sign", session) + "'/>").getBytes("UTF-8"));
    out.write("	    </form>\n".getBytes("UTF-8"));
    out.write("		<p>&nbsp; </p>\n".getBytes("UTF-8"));
    out.write("	</body>\n".getBytes("UTF-8"));
    out.write("</html>\n".getBytes("UTF-8"));
  }

  private void printSignRequest(HttpServletResponse response, String data2BSigned) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.write("<?xml version='1.0' encoding='UTF-8'?>\n".getBytes("UTF-8"));
    out.write("<sl:CreateXMLSignatureRequest xmlns:sl='http://www.buergerkarte.at/namespaces/securitylayer/1.2#'>\n".getBytes("UTF-8"));
    out.write("  <sl:KeyboxIdentifier>SecureSignatureKeypair</sl:KeyboxIdentifier>\n".getBytes("UTF-8"));
    out.write("  <sl:DataObjectInfo Structure='enveloping'>\n".getBytes("UTF-8"));
    out.write("    <sl:DataObject>\n".getBytes("UTF-8"));
    out.write(("      <sl:XMLContent>" + data2BSigned + "</sl:XMLContent>\n").getBytes("UTF-8"));
    out.write("    </sl:DataObject>\n".getBytes("UTF-8"));
    out.write("    <sl:TransformsInfo>\n".getBytes("UTF-8"));
    out.write("      <sl:FinalDataMetaInfo>\n".getBytes("UTF-8"));
    out.write("        <sl:MimeType>text/html</sl:MimeType>\n".getBytes("UTF-8"));
    out.write("      </sl:FinalDataMetaInfo>\n".getBytes("UTF-8"));
    out.write("    </sl:TransformsInfo>\n".getBytes("UTF-8"));
    out.write("  </sl:DataObjectInfo>\n".getBytes("UTF-8"));
    out.write("</sl:CreateXMLSignatureRequest>\n".getBytes("UTF-8"));
  }

  private void printFinalResponse(HttpServletResponse response, String signatureStr) throws IOException, UnsupportedEncodingException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head>");
    out.println("<title>Signieren von Beilagen: Abschluss</title>");
    out.println("<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">");
    out.println("<head>");
    out.println("<body>");
    out.println("<p>Folgende Signatur ist eingelangt:</p>");
    out.write(("<pre style=\"background-color: silver;\">" + signatureStr.replaceAll("<", "&lt;") + "</pre>").getBytes("UTF-8"));
    out.println("</body>");
    out.println("</html>");
  }

  private Iterator parseRequestAttrs(HttpServletRequest request) throws IOException
  {
    List items;
    try
    {
      DiskFileUpload upload = new DiskFileUpload();
      items = upload.parseRequest(request);
    }
    catch (FileUploadException e)
    {
      String message = "Cannot parse multipart/form-data request.";
      throw new IOException(message);
    }
    Iterator itemsIt = items.iterator();
    return itemsIt;
  }

  private Document parseDocument(String docStr) throws Exception 
  {
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    dbf.setNamespaceAware(true);
    DocumentBuilder db = dbf.newDocumentBuilder();
    ByteArrayInputStream docBAIS = new ByteArrayInputStream(docStr.getBytes("UTF-8"));
    Document doc = db.parse(docBAIS);
    return doc;
  }

  private String encodeURL(String uRL, HttpSession session)
  {
    if (uRL == null || session == null) return uRL;

    int inPos = uRL.indexOf('?');
    if (inPos == -1) inPos = uRL.indexOf('#');
    if (inPos == -1) inPos = uRL.length();
    
    String returnVal = uRL.substring(0, inPos) + ";jsessionid=" + session.getId();
    if (inPos < uRL.length()) returnVal += uRL.substring(inPos, uRL.length());
    return returnVal;
  }
  
  private String serializeElement(Element elem) throws IOException
  {
    // XML-Formatierung konfiguieren
    OutputFormat format = new OutputFormat(elem.getOwnerDocument());
    format.setLineSeparator("\n");
    format.setIndenting(false);
    format.setPreserveSpace(true);
    format.setOmitXMLDeclaration(false);
    format.setEncoding("UTF-8");

    ByteArrayOutputStream returnBAOS = new ByteArrayOutputStream();
    XMLSerializer serializer = new XMLSerializer(returnBAOS, format);
    serializer.serialize(elem);
    return returnBAOS.toString("UTF-8");
  }
  
  private String createData2BSigned(String firstName, String lastName, String hashValue)
  {
    StringBuffer data = new StringBuffer();
    data.append("<html xmlns='http://www.w3.org/1999/xhtml'>\n");
    data.append("	<head>\n");
    data.append("		<title>Formular 0815</title>\n");
    data.append("		<style type='text/css'>.silver{ background-color: silver;}</style>\n");
    data.append("	</head>\n");
    data.append("	<body>\n");
    data.append("		<h3>Formular 0815</h3>\n");
    data.append("		<p>Vorname: <code class='silver'>" + firstName + "</code></p>\n");
    data.append("		<p>Beilage: <code class='silver'>" + lastName + "</code></p>\n");
    data.append("		<h3>Beilage zu Formular 0815</h3>\n");
    data.append("		<p>Prfsumme (SHA-1, base64-kodiert): <code class='silver'>" + hashValue + "</code></p>\n");
    data.append("	</body>\n");
    data.append("</html>\n");
    return data.toString();
  }
}
