package sl12tutorial;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class SignOn extends HttpServlet
{
  static final String NS_SL_ = "http://www.buergerkarte.at/namespaces/securitylayer/1.2#"; 

   static final String BKU_URL_ = "http://127.0.0.1:3495/http-security-layer-request";
//  static final String BKU_URL_ = "http://127.0.0.1:13495/http-security-layer-request";

  public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    doGet(request, response);
  }
  
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    // Prfen, woher Aufruf kommt
    String useParam = request.getParameter("use"); 
    try
    {
	    if (useParam != null && "sign".equals(useParam))
	    {
	      // Aufruf kommt von BKU und enthlt die Personenbindung; Personenbindung in Session speichern 
	      // und CreateXMLSignature senden 
	      processSign(request, response);
	    }
	    else if (useParam != null && "done".equals(useParam))
	    {
	      // Aufruf kommt von BKU und enthlt signiertes Dokument; finale Antwortseite erzeugen und senden
	      processDone(request, response);
	    }
	    else
	    {
	      // Aufruf kommt nicht von BKU, Startseite erzeugen und senden
	      request.getSession().invalidate();
	      processInitial(request, response);
	    }
    }
    catch (Exception e)
    {
      if (!(e instanceof IOException)) throw new IOException(e.getMessage());
    }
  }
  
  private void processInitial(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    String servletURLHTTPS = "https://localhost:8443/SL12Tutorial/SignOn";

    // Neue Session beginnen
    HttpSession session = request.getSession(true);
    
    // Formular mit Befehl zum Auslesen der Personenbindung an Browser senden
    response.setStatus(200);
    response.setContentType("text/html");
    printForm(response, session, servletURLHTTPS);
  }
  
  private void processSign(HttpServletRequest request, HttpServletResponse response)
    throws Exception
  {
     String servletURL = request.getRequestURL().toString();
//     String SERVLET_URL_ = "http://localhost:18080/SL12Tutorial/SignOn";
    
    // Session abrufen
    HttpSession session = request.getSession();
    
    String xMLResponse = request.getParameter("XMLResponse");
    Document xMLResponseDoc = parseDocument(xMLResponse);
    
    // Personenbindung in Session speichern
    Element xMLResponseRoot = xMLResponseDoc.getDocumentElement();
    Element xMLContent = (Element)xMLResponseRoot.getElementsByTagNameNS(NS_SL_, "XMLContent").item(0);
    Element identityLinkRoot = (Element)xMLContent.getElementsByTagNameNS("*", "*").item(0);
    session.setAttribute("IdentityLink", identityLinkRoot);
    
    // Personenbindung prfen, wird hier nicht durchgefhrt
    
    // Befehl zum Signieren an BKU senden
    response.setStatus(307);
    response.setContentType("text/xml");
    response.setHeader("Location", encodeURL(servletURL + "?use=done", session));
    printSignRequest(response);
  }
  
  private void processDone(HttpServletRequest request, HttpServletResponse response)
    throws Exception
	{
    // Session abrufen
    HttpSession session = request.getSession();

    String xMLResponse = request.getParameter("XMLResponse");
    Document xMLResponseDoc = parseDocument(xMLResponse);

	  // XML-Sigantur in Session speichern
    Element xMLResponseRoot = xMLResponseDoc.getDocumentElement();
    Element xMLSignature = (Element)xMLResponseRoot.getElementsByTagNameNS("*", "*").item(0);
    session.setAttribute("XMLSignature", xMLSignature);

    // Signatur prfen, wird hier nicht nher ausgefhrt
    
    // Zusammengehrigkeit von Signatur und Personenbindung prfen, wird hier nicht nher ausgefhrt

	  // Abschluss
    String identityLinkStr = serializeElement((Element)session.getAttribute("IdentityLink"));
    String signatureStr = serializeElement(xMLSignature);

    response.setStatus(200);
	  response.setContentType("text/html");
    printFinalResponse(response, identityLinkStr, signatureStr);
	  
    // Session terminieren
    session.invalidate();
	}

  private void printForm(HttpServletResponse response, HttpSession session, String servletURLHTTPS) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head><title>Sign On: Start</title></head>");
    out.println("<body>");
    out.println("<form method=\"post\" action=\"" + BKU_URL_ + "\">");
    out.println("<input name=\"XMLRequest\" type=\"hidden\" value=\"&lt;?xml version='1.0' encoding='UTF-8'?>&lt;InfoboxReadRequest xmlns='http://www.buergerkarte.at/namespaces/securitylayer/1.2#'>&lt;InfoboxIdentifier>IdentityLink&lt;/InfoboxIdentifier>&lt;BinaryFileParameters ContentIsXMLEntity='true'/>&lt;/InfoboxReadRequest>\"/>");
    out.println("<input name=\"DataURL\" type=\"hidden\" value=\"" + encodeURL(servletURLHTTPS + "?use=sign", session) + "\"/>");
    out.println("<input name=\"Request absenden\" type=\"submit\"/>");
    out.println("</form>");
    out.println("</body>");
    out.println("</html>");
  }

  private void printSignRequest(HttpServletResponse response) throws IOException
  {
    String currentDate = new SimpleDateFormat("dd. MM. yyyy 'um' HH:mm:ss z").format(new Date()); 
    
    ServletOutputStream out = response.getOutputStream();
    out.write("<?xml version='1.0' encoding='UTF-8'?>\n".getBytes("UTF-8"));
    out.write("<sl:CreateXMLSignatureRequest xmlns:sl='http://www.buergerkarte.at/namespaces/securitylayer/1.2#'>\n".getBytes("UTF-8"));
    out.write("  <sl:KeyboxIdentifier>SecureSignatureKeypair</sl:KeyboxIdentifier>\n".getBytes("UTF-8"));
    out.write("  <sl:DataObjectInfo Structure='enveloping'>\n".getBytes("UTF-8"));
    out.write("    <sl:DataObject>\n".getBytes("UTF-8"));
    out.write(("      <sl:XMLContent>Ich melde mich hiermit (" + currentDate + ") zum Service XY an." + "</sl:XMLContent>\n").getBytes("UTF-8"));
    out.write("    </sl:DataObject>\n".getBytes("UTF-8"));
    out.write("    <sl:TransformsInfo>\n".getBytes("UTF-8"));
    out.write("      <sl:FinalDataMetaInfo>\n".getBytes("UTF-8"));
    out.write("        <sl:MimeType>text/plain</sl:MimeType>\n".getBytes("UTF-8"));
    out.write("      </sl:FinalDataMetaInfo>\n".getBytes("UTF-8"));
    out.write("    </sl:TransformsInfo>\n".getBytes("UTF-8"));
    out.write("  </sl:DataObjectInfo>\n".getBytes("UTF-8"));
    out.write("</sl:CreateXMLSignatureRequest>\n".getBytes("UTF-8"));
  }

  private void printFinalResponse(HttpServletResponse response, String identityLinkStr, String signatureStr) throws IOException, UnsupportedEncodingException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head>");
    out.println("<title>Sign On: Abschluss</title>");
    out.println("<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">");
    out.println("<head>");
    out.println("<body>");
    out.println("<p>Folgende Personenbindung wurde gelesen:</p>");
    out.write(("<pre style=\"background-color: silver;\">" + identityLinkStr.replaceAll("<", "&lt;") + "</pre>").getBytes("UTF-8"));
    out.println("<p>Folgende Signatur ist eingelangt:</p>");
    out.write(("<pre style=\"background-color: silver;\">" + signatureStr.replaceAll("<", "&lt;") + "</pre>").getBytes("UTF-8"));
    out.println("</body>");
    out.println("</html>");
  }

  private Document parseDocument(String docStr) throws Exception 
  {
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    dbf.setNamespaceAware(true);
    DocumentBuilder db = dbf.newDocumentBuilder();
    ByteArrayInputStream docBAIS = new ByteArrayInputStream(docStr.getBytes("UTF-8"));
    Document doc = db.parse(docBAIS);
    return doc;
  }

  private String encodeURL(String uRL, HttpSession session)
  {
    if (uRL == null || session == null) return uRL;

    int inPos = uRL.indexOf('?');
    if (inPos == -1) inPos = uRL.indexOf('#');
    if (inPos == -1) inPos = uRL.length();
    
    String returnVal = uRL.substring(0, inPos) + ";jsessionid=" + session.getId();
    if (inPos < uRL.length()) returnVal += uRL.substring(inPos, uRL.length());
    return returnVal;
  }
  
  private String serializeElement(Element elem) throws IOException
  {
    // XML-Formatierung konfiguieren
    OutputFormat format = new OutputFormat(elem.getOwnerDocument());
    format.setLineSeparator("\n");
    format.setIndenting(false);
    format.setPreserveSpace(true);
    format.setOmitXMLDeclaration(false);
    format.setEncoding("UTF-8");

    ByteArrayOutputStream returnBAOS = new ByteArrayOutputStream();
    XMLSerializer serializer = new XMLSerializer(returnBAOS, format);
    serializer.serialize(elem);
    return returnBAOS.toString("UTF-8");
  }
}
