package sl12tutorial;

import java.io.IOException;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;


public class Redirect extends HttpServlet
{
  private static final String BKU_URL_ = "http://127.0.0.1:3495/http-security-layer-request"; 
//  private static final String BKU_URL_  "http://127.0.0.1:13495/http-security-layer-request";

  public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    doGet(request, response);
  }
  
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    // Session schon vorhanden?
    HttpSession session = request.getSession(false);
    if (session == null)
    {
      // Falls nein, Startseite erzeugen und als Antwort senden
      processInitial(request, response);
    }
    else
    {
      // Falls ja, pr�fen, ob Aufruf vom Browser oder von der BKU kommt
      String useParam = request.getParameter("use"); 
      if (useParam != null && "redirecturl".equals(useParam))
      {
        // Aufruf kommt vom Browser, Warteseite bzw. Endergebnis-Seite erzeugen und als Antwort senden
        processRedirect(request, response, session);
      }
      else if (useParam != null && "dataurl".equals(useParam))
      {
        // Aufruf kommt von der BKU, XML-Befehlsantwort auswerten und <ok/> an BKU senden
        processDataURL(request, response, session);
      }
      else
      {
        processError(request, response, "Parameter \"use\" ist nicht vorhanden oder hat falschen Wert.");
        session.invalidate();
      }
    }
  }
  
  private void processInitial(HttpServletRequest request, HttpServletResponse response) throws IOException
  {
    // Gespeicherte Session abrufen
    HttpSession session = request.getSession(true);

    String servletURL = encodeURL(request.getRequestURL().toString(), session);
//    String servletURL = encodeURL("http://localhost:18080/SL12Tutorial/Redirect", session);
    
    // Startseite mit Formular f�r BKU-Aufruf erzeugen
    response.setStatus(200);
    response.setContentType("text/html");
    printForm(response, servletURL, BKU_URL_);
  }

  private void processDataURL(HttpServletRequest request, HttpServletResponse response, HttpSession session)
    throws IOException
  {
    // XML-Befehlsantwort in Session speichern
    String xMLResponse = request.getParameter("XMLResponse");
    session.setAttribute("XMLResponse", xMLResponse != null ? xMLResponse : "Parameter XMLResponse in BKU-Request nicht gefunden.");

    // <ok/> an BKU senden
    response.setStatus(200);
    response.setContentType("text/plain");
    response.getOutputStream().print("<ok/>");
  }
  
  private void processRedirect(HttpServletRequest request, HttpServletResponse response, HttpSession session)
    throws IOException
  {
    // Pr�fen, ob XML-Befehlsantwort is Session gespeichert ist
    String xMLResponse = (String) session.getAttribute("XMLResponse");
    if (xMLResponse == null)
    {
      // Falls nein, Warteseite erzeugen und senden
      
      String servletURL = encodeURL(request.getRequestURL().toString(), session);
//      String servletURL = encodeURL("http://localhost:18080/SL12Tutorial/Redirect", session);
      
      response.setStatus(200);
      response.setContentType("text/html");
      printRedirectWait(response, servletURL);
    }
    else
    {
      // Falls ja, Seite mit Endergebnis erzeugen senden
      
      response.setStatus(200);
      response.setContentType("text/html");
      printRedirectFinal(response, xMLResponse);
      
      session.invalidate();
    }
  }

  private void processError(HttpServletRequest request, HttpServletResponse response, String msg) throws IOException
  {
    response.setStatus(200);
    response.setContentType("text/plain");
    response.getOutputStream().println("Fehler: " + msg);
  }
  
  private String encodeURL(String uRL, HttpSession session)
  {
    // Java Session ID in den verwendeten URLs kodieren
    
    if (uRL == null || session == null) return uRL;

    int inPos = uRL.indexOf('?');
    if (inPos == -1) inPos = uRL.indexOf('#');
    if (inPos == -1) inPos = uRL.length();
    
    String returnVal = uRL.substring(0, inPos) + ";jsessionid=" + session.getId();
    if (inPos < uRL.length()) returnVal += uRL.substring(inPos, uRL.length());
    return returnVal;
  }
  
  private void printForm(HttpServletResponse response, String servletURL, String bkuURL) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head><title>Redirect: Start</title></head>");
    out.println("<body>");
    out.println("<form method=\"post\" action=\"" + bkuURL + "\">");
    out.println("<input name=\"XMLRequest\" type=\"hidden\" value=\"&lt;?xml version='1.0' encoding='UTF-8'?>&lt;NullOperationRequest xmlns='http://www.buergerkarte.at/namespaces/securitylayer/1.2#'/>\"/>");
    out.println("<input name=\"DataURL\" type=\"hidden\" value=\"" + servletURL + "?use=dataurl" + "\"/>");
    out.println("<input name=\"RedirectURL\" type=\"hidden\" value=\"" + servletURL + "?use=redirecturl" +  "\"/>");
    out.println("<input name=\"Request absenden\" type=\"submit\"/>");
    out.println("</form>");
    out.println("</body>");
    out.println("</html>");
  }
  
  private void printRedirectFinal(HttpServletResponse response, String xMLResponse) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head>");
    out.println("<title>Redirect: Synchronisation erfolgreich</title>");
    out.println("</head>");
    out.println("<body>");
    out.println("<p>Die Antwort von der BKU ist eingetroffen:</p>");
    xMLResponse = xMLResponse.replaceAll("<", "&lt;");
    out.println("<p><pre>" + xMLResponse + "</pre></p>");
    out.println("</body>");
    out.println("</html>");
  }

  private void printRedirectWait(HttpServletResponse response, String servletURL) throws IOException
  {
    ServletOutputStream out = response.getOutputStream();
    out.println("<html>");
    out.println("<head>");
    out.println("<title>Redirect: Warteschleife</title>");
    out.println("<meta http-equiv=\"refresh\" content=\"10; URL=" + servletURL + "?use=redirecturl\"/>");
    out.println("</head>");
    out.println("<body>");
    out.println("<p>Bitte warten. Sobald die Antwort von der BKU eingetroffen ist, wird sie hier angezeigt.</p>");
    out.println("</body>");
    out.println("</html>");
  }
}
